@@ -2,9 +2,9 @@ from __future__ import annotations
 
 from .admin_action import AdminAction
 from .block_action import BlockAction
-from .config_action import ConfigAction
 from .leave_action import LeaveAction
 from .lfg_action import LookingForGameAction
+from .record_action import RecordAction
 from .score_action import ScoreAction
 from .tasks_action import TasksAction
 from .verify_action import VerifyAction
@@ -13,9 +13,9 @@ from .watch_action import WatchAction
 __all__ = [
     "AdminAction",
     "BlockAction",
-    "ConfigAction",
     "LeaveAction",
     "LookingForGameAction",
+    "RecordAction",
     "ScoreAction",
     "TasksAction",
     "VerifyAction",
