@@ -1,20 +1,30 @@
 from __future__ import annotations
 
 import logging
-from typing import Any, Optional
+from typing import TYPE_CHECKING, Any
 
 import discord
 from discord.embeds import Embed
 
-from .. import SpellBot
-from ..models import Channel, GuildAward
-from ..operations import safe_fetch_text_channel, safe_send_channel, safe_update_embed_origin
-from ..services import GamesService
-from ..settings import Settings
-from ..utils import EMBED_DESCRIPTION_SIZE_LIMIT
-from ..views import SetupView
+from spellbot.enums import GameFormat, GameService
+from spellbot.models import Channel, ChannelDict, GuildAward, GuildAwardDict
+from spellbot.operations import (
+    safe_fetch_text_channel,
+    safe_get_partial_message,
+    safe_send_channel,
+    safe_update_embed,
+    safe_update_embed_origin,
+)
+from spellbot.services import GamesService
+from spellbot.settings import settings
+from spellbot.utils import EMBED_DESCRIPTION_SIZE_LIMIT
+from spellbot.views import SetupView
+
 from .base_action import BaseAction
 
+if TYPE_CHECKING:
+    from spellbot import SpellBot
+
 logger = logging.getLogger(__name__)
 
 
@@ -22,7 +32,7 @@ def humanize_bool(setting: bool) -> str:
     return "✅ On" if setting else "❌ Off"
 
 
-def award_line(award: dict[str, Any]) -> str:
+def award_line(award: GuildAwardDict) -> str:
     kind = "every" if award["repeating"] else "after"
     give_or_take = "take" if award["remove"] else "give"
     verified_only = "verified only" if award["verified_only"] else ""
@@ -40,7 +50,6 @@ def award_line(award: dict[str, Any]) -> str:
 class AdminAction(BaseAction):
     def __init__(self, bot: SpellBot, interaction: discord.Interaction) -> None:
         super().__init__(bot, interaction)
-        self.settings = Settings()
 
     async def _report_failure(self) -> None:
         await safe_send_channel(
@@ -49,23 +58,25 @@ class AdminAction(BaseAction):
             ephemeral=True,
         )
 
-    async def _build_channels_embeds(self) -> list[Embed]:
+    async def _build_channels_embeds(self) -> list[Embed]:  # noqa: C901
         guild = await self.services.guilds.to_dict()
         embeds: list[Embed] = []
 
         def new_embed() -> Embed:
             embed = Embed(title=f"Configuration for channels in {guild['name']}")
-            embed.set_thumbnail(url=self.settings.ICO_URL)
-            embed.color = discord.Color(self.settings.EMBED_COLOR)
+            embed.set_thumbnail(url=settings.ICO_URL)
+            embed.color = discord.Color(settings.INFO_EMBED_COLOR)
             return embed
 
         def update_channel_settings(
-            channel: dict[str, Any],
+            channel: ChannelDict,
             channel_settings: dict[str, Any],
             col: str,
         ) -> None:
-            if channel[col] != getattr(Channel, col).default.arg:
-                channel_settings[col] = channel[col]
+            is_enum = col in ("default_format", "default_service")
+            value = channel[col].value if is_enum else channel[col]
+            if value != getattr(Channel, col).default.arg:
+                channel_settings[col] = str(channel[col]) if is_enum else value
 
         all_default = True
         embed = new_embed()
@@ -78,11 +89,15 @@ class AdminAction(BaseAction):
 
             channel_settings: dict[str, Any] = {}
             update_channel_settings(channel, channel_settings, "default_seats")
+            update_channel_settings(channel, channel_settings, "default_format")
+            update_channel_settings(channel, channel_settings, "default_service")
             update_channel_settings(channel, channel_settings, "auto_verify")
             update_channel_settings(channel, channel_settings, "unverified_only")
             update_channel_settings(channel, channel_settings, "verified_only")
             update_channel_settings(channel, channel_settings, "voice_category")
             update_channel_settings(channel, channel_settings, "show_points")
+            update_channel_settings(channel, channel_settings, "require_confirmation")
+            update_channel_settings(channel, channel_settings, "voice_invite")
             if channel_settings:
                 all_default = False
                 deets = ", ".join(
@@ -120,8 +135,8 @@ class AdminAction(BaseAction):
 
         def new_embed() -> Embed:
             embed = Embed(title=f"SpellBot Player Awards for {guild['name']}")
-            embed.set_thumbnail(url=self.settings.ICO_URL)
-            embed.color = discord.Color(self.settings.EMBED_COLOR)
+            embed.set_thumbnail(url=settings.ICO_URL)
+            embed.color = discord.Color(settings.INFO_EMBED_COLOR)
             return embed
 
         has_at_least_one_award = False
@@ -156,7 +171,7 @@ class AdminAction(BaseAction):
     async def _build_setup_embed(self) -> Embed:
         guild = await self.services.guilds.to_dict()
         embed = Embed(title=f"SpellBot Setup for {guild['name']}")
-        embed.set_thumbnail(url=self.settings.ICO_URL)
+        embed.set_thumbnail(url=settings.ICO_URL)
         description = (
             "These are the current settings for SpellBot on this server."
             " Please use the buttons below, as well as the `/set` commands,"
@@ -179,7 +194,7 @@ class AdminAction(BaseAction):
             name="Create Voice Channels",
             value=humanize_bool(guild["voice_create"]),
         )
-        embed.color = discord.Color(self.settings.EMBED_COLOR)
+        embed.color = discord.Color(settings.INFO_EMBED_COLOR)
         return embed
 
     async def forget_channel(self, channel_str: str) -> None:
@@ -206,6 +221,7 @@ class AdminAction(BaseAction):
 
         embed = await games.to_embed(dm=True)
         await safe_send_channel(self.interaction, embed=embed, ephemeral=True)
+        return None
 
     async def setup(self) -> None:
         embed = await self._build_setup_embed()
@@ -231,7 +247,7 @@ class AdminAction(BaseAction):
         count: int,
         role: str,
         message: str,
-        **options: Optional[bool],
+        **options: bool | None,
     ) -> None:
         repeating = bool(options.get("repeating", False))
         remove = bool(options.get("remove", False))
@@ -274,25 +290,25 @@ class AdminAction(BaseAction):
         )
 
         embed = Embed()
-        embed.set_thumbnail(url=self.settings.ICO_URL)
+        embed.set_thumbnail(url=settings.ICO_URL)
         embed.set_author(name="Award added!")
         line = award_line(award)
         description = f"{line}\n\nYou can view all awards with the `/set awards` command."
         embed.description = description
-        embed.color = self.settings.EMBED_COLOR
+        embed.color = settings.INFO_EMBED_COLOR
         await safe_send_channel(self.interaction, embed=embed, ephemeral=True)
 
     async def award_delete(self, guild_award_id: int) -> None:
         await self.services.guilds.award_delete(guild_award_id)
         embed = Embed()
-        embed.set_thumbnail(url=self.settings.ICO_URL)
+        embed.set_thumbnail(url=settings.ICO_URL)
         embed.set_author(name="Award deleted!")
         description = "You can view all awards with the `/set awards` command."
         embed.description = description
-        embed.color = self.settings.EMBED_COLOR
+        embed.color = settings.INFO_EMBED_COLOR
         await safe_send_channel(self.interaction, embed=embed, ephemeral=True)
 
-    async def set_motd(self, message: Optional[str] = None) -> None:
+    async def set_motd(self, message: str | None = None) -> None:
         await self.services.guilds.set_motd(message)
         await safe_send_channel(self.interaction, "Message of the day updated.", ephemeral=True)
 
@@ -322,6 +338,24 @@ class AdminAction(BaseAction):
             ephemeral=True,
         )
 
+    async def set_default_format(self, format: int) -> None:
+        assert self.interaction.channel_id is not None
+        await self.services.channels.set_default_format(self.interaction.channel_id, format)
+        await safe_send_channel(
+            self.interaction,
+            f"Default format set to {GameFormat(format)} for this channel.",
+            ephemeral=True,
+        )
+
+    async def set_default_service(self, service: int) -> None:
+        assert self.interaction.channel_id is not None
+        await self.services.channels.set_default_service(self.interaction.channel_id, service)
+        await safe_send_channel(
+            self.interaction,
+            f"Default service set to {GameService(service)} for this channel.",
+            ephemeral=True,
+        )
+
     async def set_auto_verify(self, setting: bool) -> None:
         assert self.interaction.channel_id is not None
         await self.services.channels.set_auto_verify(self.interaction.channel_id, setting)
@@ -349,47 +383,121 @@ class AdminAction(BaseAction):
             ephemeral=True,
         )
 
-    async def set_channel_motd(self, message: Optional[str] = None) -> None:
+    async def set_channel_motd(self, message: str | None = None) -> None:
         assert self.interaction.channel_id is not None
-        motd = await self.services.channels.set_motd(self.interaction.channel_id, message)
+        setting = await self.services.channels.set_motd(self.interaction.channel_id, message)
         await safe_send_channel(
             self.interaction,
-            f"Message of the day for this channel has been set to: {motd}",
+            f"Message of the day for this channel has been set to: {setting}",
             ephemeral=True,
         )
 
-    async def set_channel_extra(self, message: Optional[str] = None) -> None:
+    async def set_channel_extra(self, message: str | None = None) -> None:
         assert self.interaction.channel_id is not None
-        extra = await self.services.channels.set_extra(self.interaction.channel_id, message)
+        setting = await self.services.channels.set_extra(self.interaction.channel_id, message)
         await safe_send_channel(
             self.interaction,
-            f"Extra message for this channel has been set to: {extra}",
+            f"Extra message for this channel has been set to: {setting}",
             ephemeral=True,
         )
 
     async def set_voice_category(self, value: str) -> None:
         assert self.interaction.channel_id is not None
-        name = await self.services.channels.set_voice_category(self.interaction.channel_id, value)
+        setting = await self.services.channels.set_voice_category(
+            self.interaction.channel_id, value
+        )
         await safe_send_channel(
             self.interaction,
-            f"Voice category prefix for this channel has been set to: {name}",
+            f"Voice category prefix for this channel has been set to: {setting}",
             ephemeral=True,
         )
 
     async def set_delete_expired(self, value: bool) -> None:
         assert self.interaction.channel_id is not None
-        name = await self.services.channels.set_delete_expired(self.interaction.channel_id, value)
+        setting = await self.services.channels.set_delete_expired(
+            self.interaction.channel_id, value
+        )
         await safe_send_channel(
             self.interaction,
-            f"Delete expired setting for this channel has been set to: {name}",
+            f"Delete expired setting for this channel has been set to: {setting}",
             ephemeral=True,
         )
 
     async def set_show_points(self, value: bool) -> None:
         assert self.interaction.channel_id is not None
-        name = await self.services.channels.set_show_points(self.interaction.channel_id, value)
+        setting = await self.services.channels.set_show_points(self.interaction.channel_id, value)
         await safe_send_channel(
             self.interaction,
-            f"Show points setting for this channel has been set to: {name}",
+            f"Show points setting for this channel has been set to: {setting}",
+            ephemeral=True,
+        )
+
+    async def set_require_confirmation(self, value: bool) -> None:
+        assert self.interaction.channel_id is not None
+        setting = await self.services.channels.set_require_confirmation(
+            self.interaction.channel_id, value
+        )
+        await safe_send_channel(
+            self.interaction,
+            f"Require confirmation setting for this channel has been set to: {setting}",
+            ephemeral=True,
+        )
+
+    async def set_voice_invite(self, value: bool) -> None:
+        assert self.interaction.channel_id is not None
+        setting = await self.services.channels.set_voice_invite(self.interaction.channel_id, value)
+        await safe_send_channel(
+            self.interaction,
+            f"Voice invite setting for this channel has been set to: {setting}",
+            ephemeral=True,
+        )
+
+    async def move_user(
+        self, guild_xid: int, from_user_xid: int, to_user_xid: int
+    ) -> None:  # pragma: no cover
+        if error := await self.services.users.move_user(guild_xid, from_user_xid, to_user_xid):
+            await safe_send_channel(self.interaction, f"Error: {error}", ephemeral=True)
+            return
+        await safe_send_channel(
+            self.interaction,
+            f"User {from_user_xid} has been moved to {to_user_xid}",
+            ephemeral=True,
+        )
+
+    async def set_points(self, game_id: int, player_xid: int, points: int) -> None:
+        found = await self.services.games.select(game_id)
+        if not found:
+            await safe_send_channel(
+                self.interaction,
+                "There is no game with that ID.",
+                ephemeral=True,
+            )
+            return
+
+        if not await self.services.games.players_included(player_xid):
+            await safe_send_channel(
+                self.interaction,
+                f"User <@{player_xid}> did not play in game SB{found.get('id')}.",
+                ephemeral=True,
+            )
+            return
+
+        await self.services.games.add_points(player_xid, points)
+        await self.services.games.confirm_points(player_xid)
+
+        for post in found.get("posts", []):
+            guild_xid = post["guild_xid"]
+            channel_xid = post["channel_xid"]
+            channel = await safe_fetch_text_channel(self.bot, guild_xid, channel_xid)
+            if channel:
+                message_xid = post["message_xid"]
+                message = safe_get_partial_message(channel, guild_xid, message_xid)
+                if message:
+                    embed = await self.services.games.to_embed()
+                    await safe_update_embed(message, embed=embed)
+
+        await safe_send_channel(
+            self.interaction,
+            f"Points for <@{player_xid}> for game SB{game_id} set to {points}.",
             ephemeral=True,
         )
