@@ -1,5 +1,4 @@
 import logging
-from typing import Optional
 
 import discord
 from ddtrace import tracer
@@ -7,12 +6,13 @@ from discord import app_commands
 from discord.app_commands import Choice
 from discord.ext import commands
 
-from .. import SpellBot
-from ..actions.lfg_action import LookingForGameAction
-from ..metrics import add_span_context
-from ..models import GameFormat
-from ..operations import safe_defer_interaction
-from ..utils import for_all_callbacks, is_guild
+from spellbot import SpellBot
+from spellbot.actions.lfg_action import LookingForGameAction
+from spellbot.enums import GameFormat, GameService
+from spellbot.metrics import add_span_context
+from spellbot.operations import safe_defer_interaction
+from spellbot.settings import settings
+from spellbot.utils import for_all_callbacks, is_guild
 
 logger = logging.getLogger(__name__)
 
@@ -36,21 +36,27 @@ class LookingForGameCog(commands.Cog):
     @app_commands.choices(
         format=[Choice(name=str(format), value=format.value) for format in GameFormat],
     )
+    @app_commands.choices(
+        service=[Choice(name=str(service), value=service.value) for service in GameService],
+    )
     @tracer.wrap(name="interaction", resource="lfg")
     async def lfg(
         self,
         interaction: discord.Interaction,
-        friends: Optional[str] = None,
-        seats: Optional[int] = None,
-        format: Optional[int] = None,
+        friends: str | None = None,
+        seats: int | None = None,
+        format: int | None = None,
+        service: int | None = None,
     ) -> None:
         assert interaction.guild is not None
         add_span_context(interaction)
         await safe_defer_interaction(interaction)
-        async with self.bot.guild_lock(interaction.guild.id):
-            async with LookingForGameAction.create(self.bot, interaction) as action:
-                await action.execute(friends=friends, seats=seats, format=format)
+        async with (
+            self.bot.guild_lock(interaction.guild.id),
+            LookingForGameAction.create(self.bot, interaction) as action,
+        ):
+            await action.execute(friends=friends, seats=seats, format=format, service=service)
 
 
 async def setup(bot: SpellBot) -> None:  # pragma: no cover
-    await bot.add_cog(LookingForGameCog(bot), guild=bot.settings.GUILD_OBJECT)
+    await bot.add_cog(LookingForGameCog(bot), guild=settings.GUILD_OBJECT)
