@@ -3,7 +3,7 @@ from __future__ import annotations
 import logging
 from contextlib import asynccontextmanager
 from contextvars import ContextVar
-from typing import Any, AsyncGenerator, Generic, NoReturn, Type, TypeVar
+from typing import TYPE_CHECKING, Any, Generic, NoReturn, TypeVar
 from uuid import uuid4
 
 from asgiref.sync import sync_to_async
@@ -13,11 +13,15 @@ from sqlalchemy.orm import sessionmaker
 from sqlalchemy.orm.session import Session
 from wrapt import CallableObjectProxy
 
-from .models import create_all
+from .models import create_all, reverse_all
+from .settings import settings
+
+if TYPE_CHECKING:
+    from collections.abc import AsyncGenerator
 
 logger = logging.getLogger(__name__)
 ProxiedObject = TypeVar("ProxiedObject")
-context_vars: dict["ContextLocal", ContextVar] = {}  # type: ignore
+context_vars: dict[ContextLocal, ContextVar] = {}  # type: ignore
 
 
 class ContextLocal(Generic[ProxiedObject]):
@@ -25,7 +29,7 @@ class ContextLocal(Generic[ProxiedObject]):
         context_vars[self] = ContextVar(str(uuid4()))
 
     @classmethod
-    def of_type(cls, _: Type[ProxiedObject]) -> ProxiedObject:
+    def of_type(cls, _: type[ProxiedObject]) -> ProxiedObject:
         return cls()  # type: ignore
 
     def set(self, obj: ProxiedObject) -> None:
@@ -36,10 +40,10 @@ class ContextLocal(Generic[ProxiedObject]):
         return getattr(obj, name)
 
     def __copy__(self) -> NoReturn:
-        raise NotImplementedError()
+        raise NotImplementedError
 
     def __deepcopy__(self, memo: Any) -> NoReturn:
-        raise NotImplementedError()
+        raise NotImplementedError
 
 
 class TypedProxy(Generic[ProxiedObject], CallableObjectProxy):
@@ -47,17 +51,17 @@ class TypedProxy(Generic[ProxiedObject], CallableObjectProxy):
         super().__init__(None)  # type: ignore
 
     @classmethod
-    def of_type(cls, _: Type[ProxiedObject]) -> ProxiedObject:
+    def of_type(cls, _: type[ProxiedObject]) -> ProxiedObject:
         return cls()  # type: ignore
 
     def set(self, obj: ProxiedObject) -> None:
         super().__init__(obj)  # type: ignore
 
     def __copy__(self) -> NoReturn:
-        raise NotImplementedError()
+        raise NotImplementedError
 
     def __deepcopy__(self, memo: Any) -> NoReturn:
-        raise NotImplementedError()
+        raise NotImplementedError
 
 
 engine = TypedProxy.of_type(Engine)
@@ -73,6 +77,7 @@ def initialize_connection(
     *,
     use_transaction: bool = False,
     run_migrations: bool = True,
+    worker_id: str | None = None,
 ) -> None:
     """
     Connect to the database.
@@ -93,13 +98,14 @@ def initialize_connection(
     `rollback_transaction()`. This is useful for allowing tests to run within their
     own transaction that is always rolled back.
     """
-    from .settings import Settings
-
-    settings = Settings()
+    db_url = settings.DATABASE_URL
+    if worker_id:
+        db_url += f"-{worker_id}"
+        app += f"-{worker_id}"
     if run_migrations:  # pragma: no cover
-        create_all(settings.DATABASE_URL)
+        create_all(db_url)
     engine_obj = create_engine(
-        settings.DATABASE_URL,
+        db_url,
         echo=settings.DATABASE_ECHO,
         connect_args={"application_name": app},
         isolation_level=None if use_transaction else "AUTOCOMMIT",
@@ -108,19 +114,19 @@ def initialize_connection(
 
     if use_transaction:  # pragma: no cover
         transaction_obj = connection_obj.begin()
-        transaction.set(transaction_obj)  # type: ignore
+        transaction.set(transaction_obj)
 
     db_session_maker_obj = sessionmaker(bind=connection_obj)
 
-    engine.set(engine_obj)  # type: ignore
-    connection.set(connection_obj)  # type: ignore
-    db_session_maker.set(db_session_maker_obj)  # type: ignore
+    engine.set(engine_obj)
+    connection.set(connection_obj)
+    db_session_maker.set(db_session_maker_obj)
 
 
 @sync_to_async()
 def begin_session() -> None:
     db_session = db_session_maker()
-    DatabaseSession.set(db_session)  # type: ignore
+    DatabaseSession.set(db_session)
 
 
 @sync_to_async()
@@ -147,4 +153,9 @@ def rollback_transaction() -> None:
         transaction.rollback()
     if not connection.closed:  # pragma: no cover
         connection.close()
+    print(engine.pool.status())  # noqa: T201
     engine.dispose()
+
+
+def delete_test_database(worker_id: str) -> None:
+    reverse_all(f"{settings.DATABASE_URL}-{worker_id}")
