@@ -1,4 +1,3 @@
-# type: ignore
 from __future__ import annotations
 
 from alembic import context
@@ -7,7 +6,7 @@ from sqlalchemy import engine_from_config, pool
 
 import_models()
 config = context.config
-target_metadata = Base.metadata
+target_metadata = Base.metadata  # type: ignore
 
 
 def run_migrations_offline() -> None:
@@ -15,11 +14,12 @@ def run_migrations_offline() -> None:
     Run migrations in 'offline' mode.
 
     This configures the context with just a URL and not an Engine, though an Engine
-    is acceptable here as well.  By skipping the Engine creation we don't even need a
+    is acceptable here as well. By skipping the Engine creation we don't even need a
     DBAPI to be available. Calls to context.execute() here emit the given string to
     the script output.
     """
     url = config.get_main_option("sqlalchemy.url")
+    assert url is not None
     is_sqlite = url.startswith("sqlite:")
     context.configure(
         url=url,
