@@ -1,7 +1,7 @@
 from __future__ import annotations
 
 from datetime import datetime
-from typing import TYPE_CHECKING, Any
+from typing import TYPE_CHECKING, TypedDict, cast
 
 from sqlalchemy import BigInteger, Column, DateTime, String
 from sqlalchemy.orm import relationship
@@ -9,10 +9,33 @@ from sqlalchemy.sql.expression import false, text
 from sqlalchemy.sql.schema import ForeignKey
 from sqlalchemy.sql.sqltypes import Boolean, Integer
 
+from spellbot.enums import GameFormat, GameService
+
 from . import Base, now
 
-if TYPE_CHECKING:  # pragma: no cover
-    from . import Game, Guild  # noqa
+if TYPE_CHECKING:
+    from . import Game, Guild  # noqa: F401
+
+
+class ChannelDict(TypedDict):
+    xid: int
+    created_at: datetime
+    updated_at: datetime
+    guild_xid: int
+    name: str
+    default_seats: int
+    default_format: GameFormat
+    default_service: GameService
+    auto_verify: bool
+    unverified_only: bool
+    verified_only: bool
+    motd: str
+    extra: str
+    voice_category: str
+    voice_invite: bool
+    delete_expired: bool
+    show_points: bool
+    require_confirmation: bool
 
 
 class Channel(Base):
@@ -20,11 +43,14 @@ class Channel(Base):
 
     __tablename__ = "channels"
 
-    xid = Column(
-        BigInteger,
-        primary_key=True,
-        nullable=False,
-        doc="The external Discord ID for a channel",
+    xid: int = cast(
+        int,
+        Column(
+            BigInteger,
+            primary_key=True,
+            nullable=False,
+            doc="The external Discord ID for a channel",
+        ),
     )
     created_at = Column(
         DateTime,
@@ -59,6 +85,22 @@ class Channel(Base):
         server_default=text("4"),
         doc="The default number of players that should be seated at newly created games.",
     )
+    default_format = Column(
+        Integer(),
+        default=GameFormat.COMMANDER.value,
+        server_default=text(str(GameFormat.COMMANDER.value)),
+        index=True,
+        nullable=False,
+        doc="The default Magic: The Gathering format for this channel",
+    )
+    default_service = Column(
+        Integer(),
+        default=GameService.SPELLTABLE.value,
+        server_default=text(str(GameService.SPELLTABLE.value)),
+        index=True,
+        nullable=False,
+        doc="The default service for games in this channel",
+    )
     auto_verify = Column(
         Boolean,
         nullable=False,
@@ -109,6 +151,20 @@ class Channel(Base):
         server_default=false(),
         doc="Configuration for showing points reporting on started games",
     )
+    require_confirmation = Column(
+        Boolean,
+        nullable=False,
+        default=False,
+        server_default=false(),
+        doc="Configuration for requiring confirmation on points reporting",
+    )
+    voice_invite = Column(
+        Boolean,
+        nullable=False,
+        default=False,
+        server_default=false(),
+        doc="Configuration for creating voice invites for games in this channel.",
+    )
 
     guild = relationship(
         "Guild",
@@ -122,7 +178,7 @@ class Channel(Base):
         doc="The games created in this channel",
     )
 
-    def to_dict(self) -> dict[str, Any]:
+    def to_dict(self) -> ChannelDict:
         return {
             "xid": self.xid,
             "created_at": self.created_at,
@@ -130,12 +186,16 @@ class Channel(Base):
             "guild_xid": self.guild_xid,
             "name": self.name,
             "default_seats": self.default_seats,
+            "default_format": GameFormat(cast(int, self.default_format)),
+            "default_service": GameService(cast(int, self.default_service)),
             "auto_verify": self.auto_verify,
             "unverified_only": self.unverified_only,
             "verified_only": self.verified_only,
             "motd": self.motd,
             "extra": self.extra,
             "voice_category": self.voice_category,
+            "voice_invite": self.voice_invite,
             "delete_expired": self.delete_expired,
             "show_points": self.show_points,
+            "require_confirmation": self.require_confirmation,
         }
