@@ -1,15 +1,31 @@
 from __future__ import annotations
 
 from datetime import datetime
-from typing import TYPE_CHECKING, Any
+from typing import TYPE_CHECKING, TypedDict, cast
 
-from sqlalchemy import BigInteger, Boolean, Column, DateTime, String, false
+from sqlalchemy import BigInteger, Boolean, Column, DateTime, String, false, null
 from sqlalchemy.orm import relationship
 
 from . import Base, now
 
-if TYPE_CHECKING:  # pragma: no cover
-    from . import Channel, Game, GuildAward  # noqa
+if TYPE_CHECKING:
+    from collections.abc import Iterable
+
+    from . import Channel, ChannelDict, Game, GuildAward, GuildAwardDict  # noqa: F401
+
+
+class GuildDict(TypedDict):
+    xid: int
+    created_at: datetime
+    updated_at: datetime
+    name: str
+    motd: str
+    show_links: bool
+    voice_create: bool
+    channels: list[ChannelDict]
+    awards: list[GuildAwardDict]
+    banned: bool
+    notice: str
 
 
 class Guild(Base):
@@ -17,7 +33,7 @@ class Guild(Base):
 
     __tablename__ = "guilds"
 
-    xid = Column(BigInteger, primary_key=True, nullable=False)
+    xid: int = cast(int, Column(BigInteger, primary_key=True, nullable=False))
     created_at = Column(
         DateTime,
         nullable=False,
@@ -55,6 +71,20 @@ class Guild(Base):
         server_default=false(),
         doc="Configuration for automatically created voice channels for games",
     )
+    banned = Column(
+        Boolean,
+        nullable=False,
+        default=False,
+        server_default=false(),
+        doc="If true, this guild is banned from using SpellBot",
+    )
+    notice = Column(
+        String(255),
+        doc="Notice to display to users in this guild",
+        nullable=True,
+        default=None,
+        server_default=null(),
+    )
 
     games = relationship(
         "Game",
@@ -76,7 +106,7 @@ class Guild(Base):
         doc="Awards available in this guild",
     )
 
-    def to_dict(self) -> dict[str, Any]:
+    def to_dict(self) -> GuildDict:
         return {
             "xid": self.xid,
             "created_at": self.created_at,
@@ -86,11 +116,13 @@ class Guild(Base):
             "show_links": self.show_links,
             "voice_create": self.voice_create,
             "channels": sorted(
-                [channel.to_dict() for channel in self.channels],
+                [channel.to_dict() for channel in cast("Iterable[Channel]", self.channels)],
                 key=lambda c: c["xid"],
             ),
             "awards": sorted(
-                [award.to_dict() for award in self.awards],
+                [award.to_dict() for award in cast("Iterable[GuildAward]", self.awards)],
                 key=lambda c: c["id"],
             ),
+            "banned": self.banned,
+            "notice": self.notice,
         }
