@@ -2,7 +2,7 @@ from __future__ import annotations
 
 import logging
 from asyncio import sleep
-from typing import TYPE_CHECKING, Any, Awaitable, Callable, Optional, Union, cast
+from typing import TYPE_CHECKING, Any, cast
 
 import discord
 from aiohttp.client_exceptions import ClientOSError
@@ -14,21 +14,26 @@ from .metrics import add_span_error
 from .utils import (
     CANT_SEND_CODE,
     bot_can_delete_channel,
+    bot_can_delete_message,
     bot_can_manage_channels,
     bot_can_read,
+    bot_can_read_messages,
     bot_can_reply_to,
     bot_can_role,
+    bot_can_send_messages,
     log_info,
     log_warning,
     suppress,
 )
 
 if TYPE_CHECKING:
+    from collections.abc import Awaitable, Callable
+
     from discord.abc import MessageableChannel, PrivateChannel
     from discord.guild import GuildChannel
     from discord.threads import Thread
 
-    GetChannelReturnType = Optional[Union[GuildChannel, Thread, PrivateChannel]]
+    GetChannelReturnType = GuildChannel | Thread | PrivateChannel | None
 
 logger = logging.getLogger(__name__)
 bad_users: set[int] = set()
@@ -50,15 +55,15 @@ async def retry(func: Callable[[], Awaitable[Any]]) -> Any:
 @tracer.wrap()
 async def safe_original_response(
     interaction: discord.Interaction,
-) -> Optional[discord.InteractionMessage]:
-    response: Optional[discord.InteractionMessage] = None
+) -> discord.InteractionMessage | None:
+    response: discord.InteractionMessage | None = None
     with suppress(
         DiscordException,
         ClientOSError,
         log="could fetch original response for user %(user_xid)s",
         user_xid=interaction.user.id,
     ):
-        response = await retry(lambda: interaction.original_response())
+        response = await retry(interaction.original_response)
     return response
 
 
@@ -67,21 +72,21 @@ async def safe_defer_interaction(interaction: discord.Interaction) -> None:
     with suppress(
         DiscordException,
         ClientOSError,
-        log="could defer interaction for user %(user_xid)s",
+        log="could not defer interaction for user %(user_xid)s",
         user_xid=interaction.user.id,
     ):
-        await retry(lambda: interaction.response.defer())
+        await retry(interaction.response.defer)
 
 
 @tracer.wrap()
 async def safe_fetch_user(
     client: discord.Client,
     user_xid: int,
-) -> Optional[discord.User]:
+) -> discord.User | None:
     if span := tracer.current_span():  # pragma: no cover
         span.set_tags({"user_xid": str(user_xid)})
 
-    user: Optional[discord.User]
+    user: discord.User | None
     if user := client.get_user(user_xid):
         return user
 
@@ -99,11 +104,11 @@ async def safe_fetch_user(
 async def safe_fetch_guild(
     client: discord.Client,
     guild_xid: int,
-) -> Optional[discord.Guild]:
+) -> discord.Guild | None:
     if span := tracer.current_span():  # pragma: no cover
         span.set_tags({"guild_xid": str(guild_xid)})
 
-    guild: Optional[discord.Guild]
+    guild: discord.Guild | None
     if guild := client.get_guild(guild_xid):
         return guild
 
@@ -122,7 +127,7 @@ async def safe_fetch_text_channel(
     client: discord.Client,
     guild_xid: int,
     channel_xid: int,
-) -> Optional[discord.TextChannel]:
+) -> discord.TextChannel | None:
     if span := tracer.current_span():  # pragma: no cover
         span.set_tags({"guild_xid": str(guild_xid), "channel_xid": str(channel_xid)})
 
@@ -131,7 +136,14 @@ async def safe_fetch_text_channel(
             return None
         return got
 
-    channel: Optional[discord.TextChannel] = None
+    if not (guild := await safe_fetch_guild(client, guild_xid)):
+        return None
+
+    if not bot_can_read_messages(guild):
+        logger.warning("in guild %s, no permissions to read messages", guild_xid)
+        return None
+
+    channel: discord.TextChannel | None = None
     with suppress(
         DiscordException,
         ClientOSError,
@@ -151,17 +163,17 @@ def safe_get_partial_message(
     channel: MessageableChannel,
     guild_xid: int,
     message_xid: int,
-) -> Optional[discord.PartialMessage]:
+) -> discord.PartialMessage | None:
     if span := tracer.current_span():  # pragma: no cover
         span.set_tags({"guild_xid": str(guild_xid), "message_xid": str(message_xid)})
 
-    if not hasattr(channel, "type") or getattr(channel, "type") != discord.ChannelType.text:
+    if not hasattr(channel, "type") or channel.type != discord.ChannelType.text:
         return None
 
     if not bot_can_read(channel):
         return None
 
-    message: Optional[discord.PartialMessage] = None
+    message: discord.PartialMessage | None = None
     text_channel = cast(discord.TextChannel, channel)
     with suppress(
         DiscordException,
@@ -176,14 +188,14 @@ def safe_get_partial_message(
 
 @tracer.wrap()
 async def safe_update_embed(
-    message: Union[discord.Message, discord.PartialMessage],
+    message: discord.Message | discord.PartialMessage,
     *args: Any,
     **kwargs: Any,
-) -> Optional[discord.Message]:
+) -> discord.Message | None:
     if span := tracer.current_span():  # pragma: no cover
         span.set_tags({"message_xid": str(message.id)})
 
-    updated_message: Optional[discord.Message] = None
+    updated_message: discord.Message | None = None
     with suppress(
         DiscordException,
         ClientOSError,
@@ -199,10 +211,24 @@ async def safe_update_embed(
 
 
 @tracer.wrap()
-async def safe_delete_message(message: Union[discord.Message, discord.PartialMessage]) -> bool:
+async def safe_delete_message(message: discord.Message | discord.PartialMessage) -> bool:
     if span := tracer.current_span():  # pragma: no cover
         span.set_tags({"message_xid": str(message.id)})
 
+    if not bot_can_delete_message(message):
+        maybe_guild = getattr(message, "guild", None)
+        maybe_guild_xid = getattr(maybe_guild, "id", None)
+        logger.warning(
+            (
+                "warning: in guild %s (%s), could not manage message:"
+                " no permissions to manage message: %s"
+            ),
+            maybe_guild,
+            maybe_guild_xid,
+            str(message),
+        )
+        return False
+
     success: bool = False
     with suppress(
         DiscordException,
@@ -210,7 +236,7 @@ async def safe_delete_message(message: Union[discord.Message, discord.PartialMes
         log="could not delete message %(message_xid)s",
         message_xid=message.id,
     ):
-        await retry(lambda: message.delete())
+        await retry(message.delete)
         success = True
     return success
 
@@ -244,18 +270,18 @@ async def safe_create_category_channel(
     client: discord.Client,
     guild_xid: int,
     name: str,
-) -> Optional[discord.CategoryChannel]:
+) -> discord.CategoryChannel | None:
     if span := tracer.current_span():  # pragma: no cover
         span.set_tags({"guild_xid": str(guild_xid), "name": name})
 
-    guild: Optional[discord.Guild]
+    guild: discord.Guild | None
     if not (guild := await retry(lambda: safe_fetch_guild(client, guild_xid))):
         return None
 
     if not bot_can_manage_channels(guild):
         return None
 
-    channel: Optional[discord.CategoryChannel] = None
+    channel: discord.CategoryChannel | None = None
     with suppress(
         DiscordException,
         ClientOSError,
@@ -271,16 +297,16 @@ async def safe_create_voice_channel(
     client: discord.Client,
     guild_xid: int,
     name: str,
-    category: Optional[discord.CategoryChannel] = None,
-) -> Optional[discord.VoiceChannel]:
+    category: discord.CategoryChannel | None = None,
+) -> discord.VoiceChannel | None:
     if span := tracer.current_span():  # pragma: no cover
         span.set_tags({"guild_xid": str(guild_xid), "name": name})
 
-    guild: Optional[discord.Guild]
+    guild: discord.Guild | None
     if not (guild := await retry(lambda: safe_fetch_guild(client, guild_xid))):
         return None
 
-    channel: Optional[discord.VoiceChannel] = None
+    channel: discord.VoiceChannel | None = None
     with suppress(
         DiscordException,
         ClientOSError,
@@ -292,15 +318,15 @@ async def safe_create_voice_channel(
 
 
 @tracer.wrap()
-async def safe_ensure_voice_category(
+async def safe_ensure_voice_category(  # noqa: C901
     client: discord.Client,
     guild_xid: int,
     prefix: str,
-) -> Optional[discord.CategoryChannel]:
+) -> discord.CategoryChannel | None:
     if span := tracer.current_span():  # pragma: no cover
         span.set_tags({"guild_xid": str(guild_xid), "prefix": prefix})
 
-    guild: Optional[discord.Guild]
+    guild: discord.Guild | None
     if not (guild := await retry(lambda: safe_fetch_guild(client, guild_xid))):
         return None
 
@@ -314,7 +340,7 @@ async def safe_ensure_voice_category(
     def category_name(i: int) -> str:
         return prefix if i == 0 else f"{prefix} {i + 1}"
 
-    available: Optional[discord.CategoryChannel] = None
+    available: discord.CategoryChannel | None = None
     full: list[discord.CategoryChannel] = []
     for i, cat in enumerate(
         sorted(
@@ -361,7 +387,7 @@ async def safe_delete_channel(
     if not hasattr(channel, "delete"):
         return False
 
-    channel_xid: int = getattr(channel, "id")
+    channel_xid: int = channel.id
     if span:  # pragma: no cover
         span.set_tag("channel_xid", channel_xid)
 
@@ -374,7 +400,7 @@ async def safe_delete_channel(
         channel_xid=channel_xid,
     ):
         try:
-            await retry(lambda: channel.delete())  # type: ignore
+            await retry(channel.delete)  # type: ignore - this is asserted above
             success = True
         except discord.errors.NotFound:
             logger.warning("in guild  %s, unknown channel %s", guild_xid, channel_xid)
@@ -386,7 +412,7 @@ async def safe_send_channel(
     interaction: discord.Interaction,
     *args: Any,
     **kwargs: Any,
-) -> Optional[discord.Message]:
+) -> discord.Message | None:
     if span := tracer.current_span():  # pragma: no cover
         span.set_tags(
             {
@@ -395,7 +421,7 @@ async def safe_send_channel(
             },
         )
 
-    message: Optional[discord.Message] = None
+    message: discord.Message | None = None
     with suppress(
         DiscordException,
         ClientOSError,
@@ -404,7 +430,7 @@ async def safe_send_channel(
         channel_xid=interaction.channel_id,
     ):
         await retry(lambda: interaction.response.send_message(*args, **kwargs))
-        message = await retry(lambda: interaction.original_response())
+        message = await retry(interaction.original_response)
     return message
 
 
@@ -413,7 +439,7 @@ async def safe_followup_channel(
     interaction: discord.Interaction,
     *args: Any,
     **kwargs: Any,
-) -> Optional[discord.Message]:
+) -> discord.Message | None:
     if span := tracer.current_span():  # pragma: no cover
         span.set_tags(
             {
@@ -427,7 +453,7 @@ async def safe_followup_channel(
         view_hack = MISSING if kwargs["view"] is None else kwargs["view"]
         kwargs["view"] = view_hack
 
-    message: Optional[discord.Message] = None
+    message: discord.Message | None = None
     with suppress(
         DiscordException,
         ClientOSError,
@@ -438,7 +464,7 @@ async def safe_followup_channel(
         if "content" in kwargs and kwargs["content"] is None:
             kwargs["content"] = MISSING
         await retry(lambda: interaction.followup.send(*args, **kwargs))
-        message = await retry(lambda: interaction.original_response())
+        message = await retry(interaction.original_response)
     return message
 
 
@@ -447,13 +473,18 @@ async def safe_channel_reply(
     channel: discord.TextChannel,
     *args: Any,
     **kwargs: Any,
-) -> Optional[discord.Message]:
+) -> discord.Message | None:
     guild_xid = channel.guild.id
     channel_xid = channel.id
     if span := tracer.current_span():  # pragma: no cover
         span.set_tags({"guild_xid": str(guild_xid), "channel_xid": str(channel_xid)})
-
-    message: Optional[discord.Message] = None
+    if not bot_can_send_messages(channel):
+        return log_info(
+            "in guild %(guild_xid)s, could not reply to channel %(channel_xid)s",
+            guild_xid=guild_xid,
+            channel_xid=channel_xid,
+        )
+    message: discord.Message | None = None
     with suppress(
         DiscordException,
         ClientOSError,
@@ -465,6 +496,29 @@ async def safe_channel_reply(
     return message
 
 
+@tracer.wrap()
+async def save_create_channel_invite(
+    channel: discord.abc.GuildChannel,
+    *args: Any,
+    **kwargs: Any,
+) -> discord.Invite | None:
+    guild_xid = channel.guild.id
+    channel_xid = channel.id
+    if span := tracer.current_span():  # pragma: no cover
+        span.set_tags({"guild_xid": str(guild_xid), "channel_xid": str(channel_xid)})
+
+    invite: discord.Invite | None = None
+    with suppress(
+        DiscordException,
+        ClientOSError,
+        log="in guild %(guild_xid)s, could not create invite to channel %(channel_xid)s",
+        guild_xid=guild_xid,
+        channel_xid=channel_xid,
+    ):
+        invite = await retry(lambda: channel.create_invite(*args, **kwargs))
+    return invite
+
+
 @tracer.wrap()
 async def safe_message_reply(message: discord.Message, *args: Any, **kwargs: Any) -> None:
     if span := tracer.current_span():  # pragma: no cover
@@ -482,7 +536,7 @@ async def safe_message_reply(message: discord.Message, *args: Any, **kwargs: Any
 
 @tracer.wrap()
 async def safe_send_user(
-    user: Union[discord.User, discord.Member],
+    user: discord.User | discord.Member,
     *args: Any,
     **kwargs: Any,
 ) -> None:
@@ -539,10 +593,10 @@ async def safe_send_user(
 
 @tracer.wrap()
 async def safe_add_role(
-    user_or_member: Union[discord.User, discord.Member],
+    user_or_member: discord.User | discord.Member,
     guild: discord.Guild,
     role: str,
-    remove: Optional[bool] = False,
+    remove: bool | None = False,
 ) -> None:
     if span := tracer.current_span():  # pragma: no cover
         span.set_tags(
@@ -559,7 +613,7 @@ async def safe_add_role(
 
     try:
         member = cast(
-            Optional[discord.Member],
+            discord.Member | None,
             (
                 user_or_member
                 if hasattr(user_or_member, "roles")
@@ -605,10 +659,8 @@ async def safe_add_role(
     ) as ex:
         add_span_error(ex)
         logger.exception(
-            "warning: in guild %s (%s), could not add role to member %s: %s",
+            "warning: in guild %s (%s), could not add role to member %s",
             guild.name,
             guild.id,
             str(user_or_member),
-            ex,
-            exc_info=True,
         )
