@@ -1,11 +1,10 @@
-# pylint: disable=too-many-instance-attributes
 from __future__ import annotations
 
 from .awards import AwardsService, NewAward
 from .channels import ChannelsService
-from .configs import ConfigsService
 from .games import GamesService
 from .guilds import GuildsService
+from .mirrors import MirrorsService
 from .plays import PlaysService
 from .users import UsersService
 from .verifies import VerifiesService
@@ -16,9 +15,9 @@ class ServicesRegistry:
     def __init__(self) -> None:
         self.awards = AwardsService()
         self.channels = ChannelsService()
-        self.configs = ConfigsService()
         self.games = GamesService()
         self.guilds = GuildsService()
+        self.mirrors = MirrorsService()
         self.plays = PlaysService()
         self.users = UsersService()
         self.verifies = VerifiesService()
@@ -28,9 +27,9 @@ class ServicesRegistry:
 __all__ = [
     "AwardsService",
     "ChannelsService",
-    "ConfigsService",
     "GamesService",
     "GuildsService",
+    "MirrorsService",
     "NewAward",
     "PlaysService",
     "ServicesRegistry",
