@@ -1,23 +1,25 @@
 from __future__ import annotations
 
 from datetime import datetime
-from typing import Any, Optional
+from typing import TYPE_CHECKING, cast
 
-import discord
 import pytz
 from asgiref.sync import sync_to_async
 from sqlalchemy.dialects.postgresql import insert
 from sqlalchemy.sql.expression import and_
 
-from ..database import DatabaseSession
-from ..models import Channel, Guild, GuildAward
+from spellbot.database import DatabaseSession
+from spellbot.models import Channel, Guild, GuildAward, GuildAwardDict, GuildDict
+
+if TYPE_CHECKING:
+    import discord
 
 
 class GuildsService:
-    guild: Optional[Guild] = None
+    guild: Guild | None = None
 
     @sync_to_async()
-    def upsert(self, guild: discord.Guild) -> None:
+    def upsert(self, guild: discord.Guild) -> GuildDict | None:
         name_max_len = Guild.name.property.columns[0].type.length  # type: ignore
         raw_name = getattr(guild, "name", "")
         name = raw_name[:name_max_len]
@@ -44,6 +46,27 @@ class GuildsService:
             )
             .one_or_none()
         )
+        return self.guild.to_dict() if self.guild else None
+
+    @sync_to_async()
+    def set_banned(self, banned: bool, xid: int) -> None:
+        values = {
+            "xid": xid,
+            "name": "Unknown Guild",
+            "updated_at": datetime.now(tz=pytz.utc),
+            "banned": banned,
+        }
+        upsert = insert(Guild).values(**values)
+        upsert = upsert.on_conflict_do_update(
+            index_elements=[Guild.xid],
+            index_where=Guild.xid == values["xid"],
+            set_={
+                "updated_at": upsert.excluded.updated_at,
+                "banned": upsert.excluded.banned,
+            },
+        )
+        DatabaseSession.execute(upsert, values)
+        DatabaseSession.commit()
 
     @sync_to_async()
     def select(self, guild_xid: int) -> bool:
@@ -59,10 +82,10 @@ class GuildsService:
     @sync_to_async()
     def should_voice_create(self) -> bool:
         assert self.guild
-        return self.guild.voice_create
+        return cast(bool, self.guild.voice_create)
 
     @sync_to_async()
-    def set_motd(self, message: Optional[str] = None) -> None:
+    def set_motd(self, message: str | None = None) -> None:
         if message:
             motd = message[: Guild.motd.property.columns[0].type.length]  # type: ignore
             self.guild.motd = motd  # type: ignore
@@ -73,19 +96,19 @@ class GuildsService:
     @sync_to_async()
     def toggle_show_links(self) -> None:
         assert self.guild
-        self.guild.show_links = not self.guild.show_links  # type: ignore
+        self.guild.show_links = not self.guild.show_links
         DatabaseSession.commit()
 
     @sync_to_async()
     def toggle_voice_create(self) -> None:
         assert self.guild
-        self.guild.voice_create = not self.guild.voice_create  # type: ignore
+        self.guild.voice_create = not self.guild.voice_create
         DatabaseSession.commit()
 
     @sync_to_async()
     def current_name(self) -> str:
         assert self.guild
-        return self.guild.name or ""
+        return cast(str | None, self.guild.name) or ""
 
     @sync_to_async()
     def voice_category_prefixes(self) -> list[str]:
@@ -106,7 +129,7 @@ class GuildsService:
         return [int(row[0]) for row in rows]
 
     @sync_to_async()
-    def to_dict(self) -> dict[str, Any]:
+    def to_dict(self) -> GuildDict:
         assert self.guild
         return self.guild.to_dict()
 
@@ -130,8 +153,8 @@ class GuildsService:
         count: int,
         role: str,
         message: str,
-        **options: Optional[bool],
-    ) -> dict[str, Any]:
+        **options: bool | None,
+    ) -> GuildAwardDict:
         assert self.guild
         repeating = bool(options.get("repeating", False))
         remove = bool(options.get("remove", False))
@@ -146,7 +169,7 @@ class GuildsService:
             remove=remove,
             verified_only=verified_only,
             unverified_only=unverified_only,
-        )  # type: ignore
+        )
         DatabaseSession.add(award)
         DatabaseSession.commit()
         return award.to_dict()
