@@ -1,23 +1,61 @@
-# pylint: disable=too-many-instance-attributes
 from __future__ import annotations
 
 from datetime import datetime
 from os import getenv
-from typing import Optional
+from typing import TYPE_CHECKING
 
 import pytz
 from discord import Object
-from discord.abc import Snowflake
 
 from .environment import running_in_pytest
 
+if TYPE_CHECKING:
+    from discord.abc import Snowflake
+
 
 class Settings:
-    def __init__(self, guild_xid: Optional[int] = None) -> None:
+    __slots__ = (
+        "guild_xid",
+        "CONTENT_ROOT",
+        "BOT_TOKEN",
+        "BOT_APPLICATION_ID",
+        "PORT",
+        "HOST",
+        "DEBUG_GUILD",
+        "API_BASE_URL",
+        "DD_API_KEY",
+        "DD_APP_KEY",
+        "DD_TRACE_ENABLED",
+        "DATABASE_URL",
+        "SPELLTABLE_ROOT",
+        "SPELLTABLE_CREATE",
+        "SPELLTABLE_AUTH_KEY",
+        "BOT_INVITE_LINK",
+        "INFO_EMBED_COLOR",
+        "STARTED_EMBED_COLOR",
+        "PENDING_EMBED_COLOR",
+        "EMPTY_EMBED_COLOR",
+        "DATABASE_ECHO",
+        "ADMIN_ROLE",
+        "MOD_PREFIX",
+        "MAX_PENDING_GAMES",
+        "VOICE_GRACE_PERIOD_M",
+        "VOICE_AGE_LIMIT_H",
+        "VOICE_CLEANUP_LOOP_M",
+        "VOICE_CLEANUP_BATCH",
+        "EXPIRE_GAMES_LOOP_M",
+        "EXPIRE_TIME_M",
+        "SUBSCRIBE_LINK",
+        "DONATE_LINK",
+    )
+
+    def __init__(self, guild_xid: int | None = None) -> None:
         self.guild_xid = guild_xid
 
         # content
         self.CONTENT_ROOT = "https://raw.githubusercontent.com/lexicalunit"
+        self.SUBSCRIBE_LINK = "https://www.patreon.com/lexicalunit"
+        self.DONATE_LINK = "https://ko-fi.com/lexicalunit"
 
         # application
         self.BOT_TOKEN = getenv("BOT_TOKEN")
@@ -54,7 +92,10 @@ class Settings:
             r"&permissions=2416045137"
             r"&scope=applications.commands%20bot"
         )
-        self.EMBED_COLOR = 0x5A3EFD
+        self.INFO_EMBED_COLOR = 0x5A3EFD
+        self.STARTED_EMBED_COLOR = 0xF8AE4A
+        self.PENDING_EMBED_COLOR = 0x5A3EFD
+        self.EMPTY_EMBED_COLOR = 0xCDCDCD
         self.DATABASE_ECHO = False
         self.ADMIN_ROLE = "SpellBot Admin"
         self.MOD_PREFIX = "Moderator"
@@ -71,31 +112,45 @@ class Settings:
     def workaround_over_eager_caching(self, url: str) -> str:
         return f"{url}?{datetime.now(tz=pytz.utc).date().strftime('%Y-%m-%d')}"
 
-    def queer(self) -> bool:
-        return self.guild_xid == 757455940009328670 or datetime.now(tz=pytz.utc).month == 6
+    def queer(self, guild_xid: int | None) -> bool:
+        return (
+            guild_xid
+            in [
+                757455940009328670,  # Oath of the Gaywatch
+                699775410082414733,  # Development
+            ]
+            or datetime.now(tz=pytz.utc).month == 6
+        )
 
     @property
     def THUMB_URL(self) -> str:
-        if self.queer():  # pragma: no cover
-            return self.workaround_over_eager_caching(
-                "https://user-images.githubusercontent.com/1903876/"
-                "149257079-e3efe74f-482b-4410-a0ea-dd988a4d3c63.png",
-            )
         return self.workaround_over_eager_caching(
             f"{self.CONTENT_ROOT}/spellbot/main/spellbot.png",
         )
 
+    @property
+    def QUEER_THUMB_URL(self) -> str:  # pragma: no cover
+        return self.workaround_over_eager_caching(
+            "https://user-images.githubusercontent.com/1903876/"
+            "149257079-e3efe74f-482b-4410-a0ea-dd988a4d3c63.png",
+        )
+
     @property
     def ICO_URL(self) -> str:
-        if self.queer():  # pragma: no cover
-            return self.workaround_over_eager_caching(
-                "https://user-images.githubusercontent.com/1903876/"
-                "149257564-86595c81-82a5-4558-ae40-c03d29a95d1f.png",
-            )
         return self.workaround_over_eager_caching(
             f"{self.CONTENT_ROOT}/spellbot/main/spellbot-sm.png",
         )
 
     @property
-    def GUILD_OBJECT(self) -> Optional[Snowflake]:
+    def QUEER_ICO_URL(self) -> str:  # pragma: no cover
+        return self.workaround_over_eager_caching(
+            "https://user-images.githubusercontent.com/1903876/"
+            "149257564-86595c81-82a5-4558-ae40-c03d29a95d1f.png",
+        )
+
+    @property
+    def GUILD_OBJECT(self) -> Snowflake | None:
         return Object(id=self.DEBUG_GUILD) if self.DEBUG_GUILD else None
+
+
+settings = Settings()
