@@ -4,8 +4,7 @@ import asyncio
 import logging
 import traceback
 from contextlib import AbstractContextManager
-from inspect import Traceback
-from typing import TYPE_CHECKING, Any, Optional, Type, Union, cast
+from typing import TYPE_CHECKING, Any, cast
 
 import discord
 from ddtrace import tracer
@@ -14,25 +13,27 @@ from discord.app_commands import AppCommandError, ContextMenu, NoPrivateMessage
 from discord.app_commands import Command as AppCommand
 from discord.ext.commands import AutoShardedBot
 from discord.ext.commands import Command as ExtCommand
-from discord.ui import Item
 
 from .errors import (
     AdminOnlyError,
+    GuildBannedError,
     GuildOnlyError,
     UserBannedError,
     UserUnverifiedError,
     UserVerifiedError,
 )
 from .metrics import add_span_error
-from .settings import Settings
+from .settings import settings
 
 if TYPE_CHECKING:
+    from types import TracebackType
+
     from discord.abc import MessageableChannel
+    from discord.ui import Item
 
 
 logger = logging.getLogger(__name__)
 
-
 # Discord API error code indicating that we can not send messages to this user.
 CANT_SEND_CODE = 50007
 
@@ -40,14 +41,16 @@ EMBED_DESCRIPTION_SIZE_LIMIT = 4096
 
 
 def log_warning(log: str, exec_info: bool = False, **kwargs: Any) -> None:
-    logger.warning(f"warning: discord: {log}", kwargs, exc_info=exec_info)
+    message = f"warning: discord: {log}"
+    logger.warning(message, kwargs, exc_info=exec_info)
 
 
 def log_info(log: str, exec_info: bool = False, **kwargs: Any) -> None:
-    logger.info(f"info: discord: {log}", kwargs, exc_info=exec_info)
+    message = f"info: discord: {log}"
+    logger.info(message, kwargs, exc_info=exec_info)
 
 
-def safe_permissions_for(obj: Any, *args: Any) -> Optional[discord.Permissions]:
+def safe_permissions_for(obj: Any, *args: Any) -> discord.Permissions | None:
     try:
         return obj.permissions_for(*args)
     except Exception:
@@ -66,38 +69,35 @@ def bot_can_reply_to(message: discord.Message) -> bool:
         return False
     if message.channel.guild is None:
         return False
-    try:
-        perms = safe_permissions_for(message.channel, message.channel.guild.me)
-        for req in ("send_messages",):
-            if not hasattr(perms, req) or not getattr(perms, req):
-                return False
-    except Exception:
+    perms = safe_permissions_for(message.channel, message.channel.guild.me)
+    if perms is None:
         return False
+    for req in ("send_messages",):
+        if not hasattr(perms, req) or not getattr(perms, req):
+            return False
     return True
 
 
-def bot_can_role(guild: discord.Guild, role: Optional[discord.Role] = None) -> bool:
+def bot_can_role(guild: discord.Guild, role: discord.Role | None = None) -> bool:
     if not guild.me:
         return False
     perms = guild.me.guild_permissions
     req = "manage_roles"
     if not hasattr(perms, req) or not getattr(perms, req):
         return False
-    if role is not None and role > guild.me.top_role:
-        return False
-    return True
+    return not (role is not None and role > guild.me.top_role)
 
 
 def bot_can_read(channel: MessageableChannel) -> bool:
     if not hasattr(channel, "type"):
         return False
-    channel_type = getattr(channel, "type")
+    channel_type = channel.type
     if channel_type == discord.ChannelType.private:
         return True
     guild_channel = cast(discord.abc.GuildChannel, channel)
     if not hasattr(guild_channel, "guild"):
         return True
-    guild: discord.Guild = getattr(guild_channel, "guild")
+    guild: discord.Guild = guild_channel.guild
     perms = safe_permissions_for(guild_channel, guild.me)
     for req in ("read_messages", "read_message_history"):
         if not hasattr(perms, req) or not getattr(perms, req):
@@ -110,22 +110,32 @@ def bot_can_manage_channels(guild: discord.Guild) -> bool:
         return False
     perms = guild.me.guild_permissions
     req = "manage_channels"
-    if not hasattr(perms, req) or not getattr(perms, req):
+    return hasattr(perms, req) and getattr(perms, req)
+
+
+def bot_can_delete_message(message: discord.Message | discord.PartialMessage) -> bool:
+    if not hasattr(message, "guild"):
         return False
-    return True
+    guild = message.guild
+    if not guild or not guild.me:
+        return False
+    perms = guild.me.guild_permissions
+    req = "manage_messages"
+    return hasattr(perms, req) and getattr(perms, req)
 
 
 def bot_can_delete_channel(channel: MessageableChannel) -> bool:
     if not hasattr(channel, "type"):
         return False
-    channel_type = getattr(channel, "type")
+    channel_type = channel.type
     if channel_type == discord.ChannelType.private:
         return False
     guild_channel = cast(discord.abc.GuildChannel, channel)
     if not hasattr(guild_channel, "guild"):
         return False
-    guild: discord.Guild = getattr(guild_channel, "guild")
-    if not (perms := safe_permissions_for(guild_channel, guild.me)):
+    guild: discord.Guild = guild_channel.guild
+    perms = safe_permissions_for(guild_channel, guild.me)
+    if perms is None:
         return False
     channel_id = getattr(channel, "id", None)
     logger.info("bot permissions (%s): %s", channel_id, str(perms.value))
@@ -135,44 +145,79 @@ def bot_can_delete_channel(channel: MessageableChannel) -> bool:
     return True
 
 
+def bot_can_read_messages(guild: discord.Guild) -> bool:
+    if not guild.me:
+        return False
+    perms = guild.me.guild_permissions
+    req = "read_messages"
+    return hasattr(perms, req) and getattr(perms, req)
+
+
+def bot_can_send_messages(channel: MessageableChannel) -> bool:
+    if not hasattr(channel, "type"):
+        return False
+    channel_type = channel.type
+    if channel_type == discord.ChannelType.private:
+        return False
+    guild_channel = cast(discord.abc.GuildChannel, channel)
+    if not hasattr(guild_channel, "guild"):
+        return False
+    guild: discord.Guild = guild_channel.guild
+    perms = safe_permissions_for(guild_channel, guild.me)
+    if perms is None:
+        return False
+    channel_id = getattr(channel, "id", None)
+    logger.info("bot permissions (%s): %s", channel_id, str(perms.value))
+    for req in ("send_messages",):
+        if not hasattr(perms, req) or not getattr(perms, req):
+            return False
+    return True
+
+
 def is_admin(interaction: discord.Interaction) -> bool:
     guild = getattr(interaction, "guild", None)
     channel = getattr(interaction, "channel", None)
     if not guild or not channel:
-        raise AdminOnlyError()
+        raise AdminOnlyError
     if interaction.user.id == guild.owner_id:
         return True
-    if (perms := safe_permissions_for(channel, interaction.user)) and perms.administrator:
+    perms = safe_permissions_for(channel, interaction.user)
+    if perms is not None and perms.administrator:
         return True
     if not hasattr(interaction.user, "roles"):
-        raise AdminOnlyError()
+        raise AdminOnlyError
     author_roles = interaction.user.roles  # type: ignore
-    settings = Settings()
     has_admin_role = any(
         role.name == settings.ADMIN_ROLE for role in cast(list[discord.Role], author_roles)
     )
     if not has_admin_role:
-        raise AdminOnlyError()
+        raise AdminOnlyError
     return True
 
 
 def is_guild(interaction: discord.Interaction) -> bool:
     guild = getattr(interaction, "guild", None)
     if guild is None:
-        raise GuildOnlyError()
+        raise GuildOnlyError
     return True
 
 
+def is_mod(interaction: discord.Interaction) -> bool:
+    guild = getattr(interaction, "guild", None)
+    channel = getattr(interaction, "channel", None)
+    return user_can_moderate(interaction.user, guild, channel)
+
+
 def user_can_moderate(
-    author: Optional[Union[discord.User, discord.Member]],
-    guild: Optional[discord.Guild],
-    channel: Optional[MessageableChannel],
+    author: discord.User | discord.Member | None,
+    guild: discord.Guild | None,
+    channel: MessageableChannel | None,
 ) -> bool:
     if not guild or not channel or not author:
         return False
     if not hasattr(author, "id"):
         return False
-    author_id = author.id  # type: ignore
+    author_id = author.id
     if author_id == guild.owner_id:
         return True
     if not hasattr(author, "roles"):
@@ -181,7 +226,6 @@ def user_can_moderate(
     if (perms := safe_permissions_for(channel, member)) and perms.administrator:
         return True
     author_roles = author.roles  # type: ignore
-    settings = Settings()
     return any(
         role.name == settings.ADMIN_ROLE or role.name.startswith(settings.MOD_PREFIX)
         for role in cast(list[discord.Role], author_roles)
@@ -205,7 +249,9 @@ class suppress(AbstractContextManager[None]):
     be able to understand that code following the context is reachable.
     """
 
-    def __init__(self, *exceptions: Type[Exception], log: str, **kwargs: Any) -> None:
+    __slots__ = ("_exceptions", "_log", "_kwargs")
+
+    def __init__(self, *exceptions: type[Exception], log: str, **kwargs: Any) -> None:
         self._exceptions = exceptions
         self._log = log
         self._kwargs = kwargs
@@ -213,7 +259,12 @@ class suppress(AbstractContextManager[None]):
     def __enter__(self) -> None:
         pass
 
-    def __exit__(self, exctype: Type[Exception], excinst: Exception, exctb: Traceback) -> bool:
+    def __exit__(
+        self,
+        exctype: type[BaseException] | None,
+        excinst: BaseException | None,
+        exctb: TracebackType | None,
+    ) -> bool:
         if captured := exctype is not None and issubclass(exctype, self._exceptions):
             log_warning(self._log, exec_info=True, **self._kwargs)
             if span := tracer.current_span():  # pragma: no cover
@@ -234,7 +285,7 @@ def for_all_callbacks(decorator: Any) -> Any:
     def decorate(cls: Any) -> Any:
         for attr in cls.__dict__:
             method = getattr(cls, attr)
-            if isinstance(method, (AppCommand, ExtCommand)):
+            if isinstance(method, AppCommand | ExtCommand):
                 setattr(cls, attr, decorator(method))
 
         return cls
@@ -253,6 +304,8 @@ async def handle_interaction_errors(interaction: discord.Interaction, error: Exc
         return await safe_send_user(interaction.user, "This command is not supported in DMs.")
     if isinstance(error, UserBannedError):
         return await safe_send_user(interaction.user, "You have been banned from using SpellBot.")
+    if isinstance(error, GuildBannedError):
+        return await safe_send_user(interaction.user, "You have been banned from using SpellBot.")
     if isinstance(error, UserUnverifiedError):
         return await safe_send_user(interaction.user, "Only verified users can do that here.")
     if isinstance(error, UserVerifiedError):
@@ -261,35 +314,37 @@ async def handle_interaction_errors(interaction: discord.Interaction, error: Exc
     add_span_error(error)
     ref = (
         f"command `{interaction.command.qualified_name}`"
-        if interaction.command is not None and isinstance(interaction, (AppCommand, ExtCommand))
+        if interaction.command is not None and isinstance(interaction, AppCommand | ExtCommand)
         else f"component `{interaction.command.qualified_name}`"
         if interaction.command is not None and isinstance(interaction, ContextMenu)
         else f"interaction `{interaction.id}`"
     )
     logger.error("error: unhandled exception in %s: %s: %s", ref, error.__class__.__name__, error)
     traceback.print_tb(error.__traceback__)
+    return None
 
 
 async def handle_view_errors(
     interaction: discord.Interaction,
     error: Exception,
     item: Item[Any],
-) -> None:
+) -> None:  # pragma: no cover
     return await handle_interaction_errors(interaction, error)
 
 
-async def handle_command_errors(interaction: discord.Interaction, error: AppCommandError) -> None:
+async def handle_command_errors(
+    interaction: discord.Interaction, error: AppCommandError
+) -> None:  # pragma: no cover
     return await handle_interaction_errors(interaction, error)
 
 
-async def load_extensions(bot: AutoShardedBot, do_sync: bool = False) -> None:
+async def load_extensions(bot: AutoShardedBot, do_sync: bool = False) -> None:  # pragma: no cover
     from .cogs import load_all_cogs
 
-    settings = Settings()
     guild = settings.GUILD_OBJECT
 
     if do_sync:
-        if guild:  # pragma: no cover
+        if guild:
             logger.info("syncing commands to debug guild: %s", guild.id)
         else:
             logger.info("syncing global commands")
